from __future__ import annotations
from dataclasses import dataclass, field
from typing import List, Optional
from witsml21.abstract_object import AbstractObject
from witsml21.data_object_reference import DataObjectReference
from witsml21.length_measure import LengthMeasure
from witsml21.tubular_assembly import TubularAssembly
from witsml21.tubular_component import TubularComponent
from witsml21.tubular_osduintegration import TubularOsduintegration
from witsml21.tubular_umbilical import TubularUmbilical

__NAMESPACE__ = "http://www.energistics.org/energyml/data/witsmlv2"


@dataclass
class Tubular(AbstractObject):
    """Used to capture information about the configuration of a drill string.

    For information about a use of this configuration, See the BhaRun
    object . This object is uniquely identified within the context of
    one wellbore object.

    :ivar mixed_string: Flag indicating the assembly is a mixed string.
        The length of the assembly may be made up of joints with
        different tensile strengths, or collapse resistance and yield
        strengths.
    :ivar nominal_diameter: Nominal size (diameter) describing the whole
        assembly, e.g., 9.625", 12.25".
    :ivar tubular_osduintegration: Information about a Tubular that is
        relevant for OSDU integration but does not have a natural place
        in a Tubular object.
    :ivar tubular_umbilical:
    :ivar type_tubular_assy: Type of tubular assembly.
    :ivar valve_float: Is float valve present?  Values are "true" (or
        "1") and "false" (or "0").
    :ivar source_nuclear: Is nuclear tool present?  Values are "true"
        (or "1") and "false" (or "0").
    :ivar dia_hole_assy: Maximum hole size generated by the assembly.
    :ivar tubular_component:
    :ivar wellbore:
    """
    class Meta:
        namespace = "http://www.energistics.org/energyml/data/witsmlv2"

    mixed_string: Optional[bool] = field(
        default=None,
        metadata={
            "name": "MixedString",
            "type": "Element",
        }
    )
    nominal_diameter: Optional[LengthMeasure] = field(
        default=None,
        metadata={
            "name": "NominalDiameter",
            "type": "Element",
        }
    )
    tubular_osduintegration: Optional[TubularOsduintegration] = field(
        default=None,
        metadata={
            "name": "TubularOSDUIntegration",
            "type": "Element",
        }
    )
    tubular_umbilical: List[TubularUmbilical] = field(
        default_factory=list,
        metadata={
            "name": "TubularUmbilical",
            "type": "Element",
        }
    )
    type_tubular_assy: Optional[TubularAssembly] = field(
        default=None,
        metadata={
            "name": "TypeTubularAssy",
            "type": "Element",
            "required": True,
        }
    )
    valve_float: Optional[bool] = field(
        default=None,
        metadata={
            "name": "ValveFloat",
            "type": "Element",
        }
    )
    source_nuclear: Optional[bool] = field(
        default=None,
        metadata={
            "name": "SourceNuclear",
            "type": "Element",
        }
    )
    dia_hole_assy: Optional[LengthMeasure] = field(
        default=None,
        metadata={
            "name": "DiaHoleAssy",
            "type": "Element",
        }
    )
    tubular_component: List[TubularComponent] = field(
        default_factory=list,
        metadata={
            "name": "TubularComponent",
            "type": "Element",
        }
    )
    wellbore: Optional[DataObjectReference] = field(
        default=None,
        metadata={
            "name": "Wellbore",
            "type": "Element",
            "required": True,
        }
    )
